"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _POSTAG = _interopRequireDefault(require("./POSTAG"));

var _Tokenizer = _interopRequireDefault(require("./Tokenizer"));

var _Optimizer = _interopRequireDefault(require("./Optimizer"));

/**
 * 分词器接口
 *
 * @author 老雷<leizongmin@gmail.com>
 * @author 林一二<linonetwo012@gmail.com>
 */

/**
 * 创建分词器接口
*/
var Segment =
/*#__PURE__*/
function () {
  function Segment() {
    var _this = this;

    (0, _classCallCheck2.default)(this, Segment);
    (0, _defineProperty2.default)(this, "use", function (Module) {
      // 传入列表的话就递归调用
      if (Array.isArray(Module)) {
        Module.forEach(_this.use);
      } else {
        // 初始化并注册模块
        if (typeof Module.init === 'function') {
          Module.init(_this);

          _this.modules[Module.type].push(Module);
        } else {
          var module = new Module(_this);

          _this.modules[module.type].push(module);
        }
      }

      return _this;
    });
    (0, _defineProperty2.default)(this, "loadDict", function (dict) {
      var type = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'TABLE';
      var convertToLower = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      if (Array.isArray(dict)) {
        dict.forEach(function (d) {
          return _this.loadDict(d);
        });
      } else {
        // 初始化词典
        if (!_this.DICT[type]) _this.DICT[type] = {};
        if (!_this.DICT["".concat(type, "2")]) _this.DICT["".concat(type, "2")] = {};
        var TABLE = _this.DICT[type]; // 词典表  '词' => {属性}

        var TABLE2 = _this.DICT["".concat(type, "2")]; // 词典表  '长度' => '词' => 属性
        // 导入数据


        dict.split(/\r?\n/).map(function (line) {
          if (convertToLower) return line.toLowerCase();
          return line;
        }).forEach(function (line) {
          var blocks = line.split('|');

          if (blocks.length > 2) {
            var w = blocks[0].trim();
            var p = Number(blocks[1]);
            var f = Number(blocks[2]); // 一定要检查单词是否为空，如果为空会导致Bug

            if (w.length > 0) {
              TABLE[w] = {
                f: f,
                p: p
              };
              if (!TABLE2[w.length]) TABLE2[w.length] = {};
              TABLE2[w.length][w] = TABLE[w];
            }
          }
        });
      }

      return _this;
    });
    (0, _defineProperty2.default)(this, "getDict", function (type) {
      return _this.DICT[type];
    });
    (0, _defineProperty2.default)(this, "loadSynonymDict", function (dict) {
      if (Array.isArray(dict)) {
        dict.forEach(function (d) {
          return _this.loadSynonymDict(d);
        });
      } else {
        var type = 'SYNONYM'; // 初始化词典

        if (!_this.DICT[type]) _this.DICT[type] = {};
        var TABLE = _this.DICT[type]; // 词典表  '同义词' => '标准词'
        // 导入数据

        dict.split(/\r?\n/).map(function (line) {
          return line.split(',');
        }).forEach(function (blocks) {
          if (blocks.length > 1) {
            var n1 = blocks[0].trim();
            var n2 = blocks[1].trim();
            TABLE[n1] = n2;

            if (TABLE[n2] === n1) {
              delete TABLE[n2];
            }
          }
        });
      }

      return _this;
    });
    (0, _defineProperty2.default)(this, "loadStopwordDict", function (dict) {
      if (Array.isArray(dict)) {
        dict.forEach(function (d) {
          return _this.loadStopwordDict(d);
        });
      } else {
        var type = 'STOPWORD'; // 初始化词典

        if (!_this.DICT[type]) _this.DICT[type] = {};
        var TABLE = _this.DICT[type]; // 词典表  '同义词' => '标准词'
        // 导入数据

        dict.split(/\r?\n/).map(function (line) {
          return line.trim();
        }).forEach(function (line) {
          if (line) {
            TABLE[line] = true;
          }
        });
      }

      return _this;
    });
    (0, _defineProperty2.default)(this, "doSegment", function (text, options) {
      var me = _this;
      options = options || {};
      var ret = []; // 将文本按照换行符分割成多段，并逐一分词

      text.replace(/\r/g, '\n') // 用换行符和空格把长文本切小，以减小传入中间件的数组长度
      .split(/\n+/).forEach(function (section) {
        var section = section.trim();
        if (section.length < 1) return; // ======================================
        // 分词

        var sret = me.tokenizer.split(section, me.modules.tokenizer); // 优化

        sret = me.optimizer.doOptimize(sret, me.modules.optimizer); // ======================================
        // 连接分词结果

        if (sret.length > 0) ret = ret.concat(sret);
      }); // 去除标点符号

      if (options.stripPunctuation) {
        ret = ret.filter(function (item) {
          return item.p !== _POSTAG.default.D_W;
        });
      } // 转换同义词


      function convertSynonym(list) {
        var count = 0;
        var TABLE = me.getDict('SYNONYM');
        list = list.map(function (item) {
          if (item.w in TABLE) {
            count++;
            return {
              w: TABLE[item.w],
              p: item.p
            };
          }

          return item;
        });
        return {
          count: count,
          list: list
        };
      }

      if (options.convertSynonym) {
        do {
          var result = convertSynonym(ret);
          ret = result.list;
        } while (result.count > 0);
      } // 去除停止符


      if (options.stripStopword) {
        var STOPWORD = me.getDict('STOPWORD');
        ret = ret.filter(function (item) {
          return !(item.w in STOPWORD);
        });
      } // 仅返回单词内容


      if (options.simple) {
        ret = ret.map(function (item) {
          return item.w;
        });
      }

      return ret;
    });
    this.POSTAG = _POSTAG.default; // 词性

    this.DICT = {}; // 词典表

    this.modules = {
      tokenizer: [],
      // 分词模块
      optimizer: [] // 优化模块

    };
    this.tokenizer = new _Tokenizer.default(this);
    this.optimizer = new _Optimizer.default(this);
  }
  /**
  * 载入分词模块
  *
  * @param {String|Array|Object} module 模块名称(数组)或模块对象
  * @return {Segment}
  */


  (0, _createClass2.default)(Segment, [{
    key: "toString",

    /**
    * 将单词数组连接成字符串
    *
    * @param {Array} words 单词数组
    * @return {String}
    */
    value: function toString(words) {
      return words.map(function (item) {
        return item.w;
      }).join('');
    }
    /**
    * 根据某个单词或词性来分割单词数组
    *
    * @param {Array} words 单词数组
    * @param {Number|String} s 用于分割的单词或词性
    * @return {Array}
    */

  }, {
    key: "split",
    value: function split(words, s) {
      var ret = [];
      var lasti = 0;
      var i = 0;
      var f = typeof s === 'string' ? 'w' : 'p';

      while (i < words.length) {
        if (words[i][f] === s) {
          if (lasti < i) ret.push(words.slice(lasti, i));
          ret.push(words.slice(i, i + 1));
          i++;
          lasti = i;
        } else {
          i++;
        }
      }

      if (lasti < words.length - 1) {
        ret.push(words.slice(lasti, words.length));
      }

      return ret;
    }
    /**
    * 在单词数组中查找某一个单词或词性所在的位置
    *
    * @param {Array} words 单词数组
    * @param {Number|String} s 要查找的单词或词性
    * @param {Number} cur 开始位置
    * @return {Number} 找不到，返回-1
    */

  }, {
    key: "indexOf",
    value: function indexOf(words, s, cur) {
      cur = isNaN(cur) ? 0 : cur;
      var f = typeof s === 'string' ? 'w' : 'p';

      while (cur < words.length) {
        if (words[cur][f] === s) return cur;
        cur++;
      }

      return -1;
    }
  }]);
  return Segment;
}();

exports.default = Segment;