"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _BaseModule = require("./BaseModule");

// 邮箱地址中允许出现的字符
// 参考：http://www.cs.tut.fi/~jkorpela/rfc/822addr.html
var _EMAILCHAR = '!"#$%&\'*+-/0123456789=?@ABCDEFGHIJKLMNOPQRSTUVWXYZ^_`abcdefghijklmnopqrstuvwxyz{|}~.'.split('');

var EMAILCHAR = {};

for (var i in _EMAILCHAR) {
  EMAILCHAR[_EMAILCHAR[i]] = 1;
}

var EmailOptimizer =
/*#__PURE__*/
function (_Optimizer) {
  (0, _inherits2.default)(EmailOptimizer, _Optimizer);

  function EmailOptimizer() {
    (0, _classCallCheck2.default)(this, EmailOptimizer);
    return (0, _possibleConstructorReturn2.default)(this, (0, _getPrototypeOf2.default)(EmailOptimizer).apply(this, arguments));
  }

  (0, _createClass2.default)(EmailOptimizer, [{
    key: "doOptimize",
    value: function doOptimize(words) {
      var POSTAG = this.segment.POSTAG; // debug(words);

      var i = 0;
      var ie = words.length - 1;
      var addr_start = false;
      var has_at = false;

      while (i < ie) {
        var word = words[i];
        var is_ascii = !!(word.p === POSTAG.A_NX || word.p === POSTAG.A_M && word.w.charCodeAt(0) < 128); // 如果是外文字符或者数字，符合电子邮件地址开头的条件

        if (addr_start === false && is_ascii) {
          addr_start = i;
          i++;
          continue;
        } else {
          // 如果遇到@符号，符合第二个条件
          if (has_at === false && word.w === '@') {
            has_at = true;
            i++;
            continue;
          } // 如果已经遇到过@符号，且出现了其他字符，则截取邮箱地址


          if (has_at !== false && words[i - 1].w != '@' && is_ascii === false && !(word.w in EMAILCHAR)) {
            var mailws = words.slice(addr_start, i);
            words.splice(addr_start, mailws.length, {
              w: EmailOptimizer.toEmailAddress(mailws),
              p: POSTAG.URL
            });
            i = addr_start + 1;
            ie -= mailws.length - 1;
            addr_start = false;
            has_at = false;
            continue;
          } // 如果已经开头


          if (addr_start !== false && (is_ascii || word.w in EMAILCHAR)) {
            i++;
            continue;
          }
        } // 移到下一个词


        addr_start = false;
        has_at = false;
        i++;
      } // 检查剩余部分


      if (addr_start && has_at && words[ie]) {
        var word = words[ie];
        var is_ascii = !!(word.p === POSTAG.A_NX || word.p === POSTAG.A_M && word.w in EMAILCHAR);

        if (is_ascii) {
          var mailws = words.slice(addr_start, words.length);
          words.splice(addr_start, mailws.length, {
            w: EmailOptimizer.toEmailAddress(mailws),
            p: POSTAG.URL
          });
        }
      }

      return words;
    }
    /**
    * 根据一组单词生成邮箱地址
    *
    * @param {array} words 单词数组
    * @return {string}
    */

  }], [{
    key: "toEmailAddress",
    value: function toEmailAddress(words) {
      var ret = words[0].w;

      for (var i = 1, word; word = words[i]; i++) {
        ret += word.w;
      }

      return ret;
    }
  }]);
  return EmailOptimizer;
}(_BaseModule.Optimizer);

exports.default = EmailOptimizer;